local mod = ...



-- Assets -------------------------------------------------------------------------------------
mod:registerAssetId("models/Hunt1.fbx/Prefab/Bear", "PREFAB_HUNT_BEAR")
mod:registerAssetId("models/Hunt1.fbx/Prefab/Deer", "PREFAB_HUNT_DEER")
mod:registerAssetId("models/Hunt1.fbx/Prefab/Pheasant", "PREFAB_HUNT_PHEASANT")



-- Resource: Berries (OVERRIDE) -------------------------------------------------------------------------------------
mod:registerAssetId("icons/Game.png", "ICON_GAME")

mod:override({
	Id = "BERRIES",
	ResourceName = "RESOURCE_GAME_NAME",
	Icon = "ICON_GAME",
	ResourceTypeList = { "FOOD", "GRANARY" },
	IsTradable = true, -- available to store in the warehouse
	ResourceLayoutType = "CRATES",
	VillagerBuyingPrice = { Resource = "GOLD", Quantity = 6 },
    DisplayInInventory = true,
    DisplayInToolbar = false
})



-- Resource Container -------------------------------------------------------------------------------------
mod:registerPrefabComponent( "models/Hunt1.fbx/Prefab/Bear", {
    DataType = "COMP_RESOURCE_CONTAINER",
    ResourceData = "BERRIES",
    IsReplenishable = false,
    ResourceValue = 4.0, -- how many are gathered with each pick
    AvailableQuantity = 1, -- maximum container stock
    Radius = 18, -- distance from container that the villager will stand to pick
    IsDestroyWhenEmpty = true
})

mod:registerPrefabComponent( "models/Hunt1.fbx/Prefab/Deer", {
    DataType = "COMP_RESOURCE_CONTAINER",
    ResourceData = "BERRIES",
    IsReplenishable = true,
    ReplenishDurationInDays = 28,
    ReplenishQuantity = 2,
    ResourceValue = 2.0, -- how many are gathered with each pick
    AvailableQuantity = 1, -- maximum container stock
    Radius = 18, -- distance from container that the villager will stand to pick
    IsDestroyWhenEmpty = false
})

mod:registerPrefabComponent( "models/Hunt1.fbx/Prefab/Pheasant", {
    DataType = "COMP_RESOURCE_CONTAINER",
    ResourceData = "BERRIES",
    IsReplenishable = true,
    ReplenishDurationInDays = 28,
    ReplenishQuantity = 1,
    ResourceValue = 1.0, -- how many are gathered with each pick
    AvailableQuantity = 1, -- maximum container stock
    Radius = 14, -- distance from container that the villager will stand to pick
    IsDestroyWhenEmpty = false
})



-- Plantable -------------------------------------------------------------------------------------
mod:registerPrefabComponent("models/Hunt1.fbx/Prefab/Bear", { DataType = "COMP_PLANTABLE", MinSpaceBetweenPlant = 40.0 })
mod:registerPrefabComponent("models/Hunt1.fbx/Prefab/Bear", { DataType = "COMP_TREE" })

mod:register({
    DataType = "PLANTABLE",
    Id = "PLANTABLE_HUNT_BEAR",
    Name = "PLANTABLE_HUNT_BEAR_NAME",
    PlantPrefab = "PREFAB_HUNT_BEAR",
    MinimumScale = 0.85, -- min size at full growth
    MaximumScale = 1.15 -- max size at full growth
})

mod:registerPrefabComponent("models/Hunt1.fbx/Prefab/Deer", { DataType = "COMP_PLANTABLE", MinSpaceBetweenPlant = 40.0 })
mod:registerPrefabComponent("models/Hunt1.fbx/Prefab/Deer", { DataType = "COMP_TREE" })

mod:register({
    DataType = "PLANTABLE",
    Id = "PLANTABLE_HUNT_DEER",
    Name = "PLANTABLE_HUNT_DEER_NAME",
    PlantPrefab = "PREFAB_HUNT_DEER",
    MinimumScale = 0.95, -- min size at full growth
    MaximumScale = 1.05 -- max size at full growth
})

mod:registerPrefabComponent("models/Hunt1.fbx/Prefab/Pheasant", { DataType = "COMP_PLANTABLE", MinSpaceBetweenPlant = 40.0 })
mod:registerPrefabComponent("models/Hunt1.fbx/Prefab/Pheasant", { DataType = "COMP_TREE" })

mod:register({
    DataType = "PLANTABLE",
    Id = "PLANTABLE_HUNT_PHEASANT",
    Name = "PLANTABLE_HUNT_PHEASANT_NAME",
    PlantPrefab = "PREFAB_HUNT_PHEASANT",
    MinimumScale = 0.95, -- min size at full growth
    MaximumScale = 1.05 -- max size at full growth
})



-- Building: Gathering Camp (OVERRIDE) -------------------------------------------------------------------------------------
-- Turn the Gathering camp into a Hunter's hut
mod:override({
    Id = "BUILDING_GATHERING_CAMP",
    Name = "BUILDING_HUNTERS_HUT_NAME",
    AssetCoreBuildingPart = "BUILDING_PART_HUNTERS_HUT"
})



-- Building: Hunter's Hut (OVERRIDE) -------------------------------------------------------------------------------------
-- Hide the real hunter's hut
mod:override({
    Id = "BUILDING_HUNTERS_HUT",
    IsHidden = true,
    IsManuallyUnlocked = false,
    VillagerRequired = {
        Status = "SERF",
        Quantity = 8
    }
})



-- Unlockable: Hunter's Hut (OVERRIDE) -------------------------------------------------------------------------------------
mod:override({
    Id = "UNLOCKABLE_KINGDOM_HUNTERS_HUT",
    EstateInfluenceCostList = {
        {
            Estate = "KINGDOM",
            Quantity = 0
        }
    },
    PrerequisiteUpgrades = {}
})



-- Kingdom estate unlock list (OVERRIDE) -------------------------------------------------------------------------------------
-- Rebuild the list without the Hunter's hut
mod:override({
    Id = "KINGDOM",
    EstateProgressList = {
        {
            SplendorRequired = 0,
            VillagerStatusRequired = {
                Status = "SERF",
                Quantity = 10
            },
            UpgradeList = {
                "UNLOCKABLE_KINGDOM_WOODEN_KEEP",
                "UNLOCKABLE_KINGDOM_WOODEN_WALL",
                "UNLOCKABLE_KINGDOM_LARGE_STONE_WALL"
            }
        },
        {
            SplendorRequired = 15,
            VillagerStatusRequired = {
                Status = "COMMONER",
                Quantity = 20
            },
            UpgradeList = {
                "UNLOCKABLE_KINGDOM_WOODEN_KEEP_DECORATIVE",
                "UNLOCKABLE_KINGDOM_WEAPONSMITH",
            }
        },
        {
            SplendorRequired = 40,
            VillagerStatusRequired = {
                Status = "CITIZEN",
                Quantity = 10
            },
            UpgradeList = {
                "UNLOCKABLE_KINGDOM_TRADE_BONUS",
            }
        },
    }
})



-- Building Function: Hunt Wild Game -------------------------------------------------------------------------------------
mod:register({
    DataType = "BUILDING_FUNCTION_WORKPLACE",
    Id = "BUILDING_FUNCTION_HUNT_WILD_GAME",
    Name = "BUILDING_FUNCTION_HUNT_WILD_GAME_NAME",
    WorkerCapacity = 2,
    RelatedJob = { 
        Job = "HUNTER", 
        Behavior = "BEHAVIOR_GATHER"
    },
    InputInventoryCapacity = {
		{ Resource = "BERRIES", Quantity = 1 }, 
	},
    ResourceListNeeded = {
		{ Resource = "BERRIES", Quantity = 1 }, 
	},
	ResourceProduced = {
        { Resource = "BERRIES", Quantity = 1 },
    }
})



-- Building Function: Hunter's Hut (OVERRIDE) -------------------------------------------------------------------------------------
mod:override({
    Id = "BUILDING_FUNCTION_HUNTERS_HUT",
    Name = "BUILDING_FUNCTION_HUNTERS_HUT_NAME"
})



-- Building Function: Hunter ASSIGNABLE -------------------------------------------------------------------------------------
mod:register({
    DataType = "BUILDING_FUNCTION_ASSIGNABLE",
    Id = "BUILDING_FUNCTION_ASSIGNABLE_HUNTER",
    AuthorizedFunctionList = "BUILDING_FUNCTION_LIST_HUNTER"
})

mod:register({
    DataType = "BUILDING_FUNCTION_LIST",
    Id = "BUILDING_FUNCTION_LIST_HUNTER",
    AssetFunctionList = {
        "BUILDING_FUNCTION_HUNT_WILD_GAME",
		"BUILDING_FUNCTION_HUNTERS_HUT",
    }
})



-- Job: Hunter (OVERRIDE) -------------------------------------------------------------------------------------
-- Allow hunting anywhere on the map
mod:override({
    Id = "HUNTER",
    RelatedZone = ""
})



-- Building Part: Hunter's Hut (OVERRIDE) -------------------------------------------------------------------------------------
mod:override({
    Id = "BUILDING_PART_HUNTERS_HUT",
    AssetBuildingFunction = "BUILDING_FUNCTION_ASSIGNABLE_HUNTER"
})



-- Starting Tile Resource Requirement (OVERRIDE) -------------------------------------------------------------------------------------
-- Removes requirement for berries in the starting tile
mod:override({
    Id = "DEFAULT_BALANCING",
    StartingTileResourceRequirement = {
        {
            Resource = "STONE",
            Quantity = 1
        },
        {
            Resource = "WOOD",
            Quantity = 40
        },
    }

})



-- registering map files
mod:registerAssetId("maps/heightmap.png", "HEIGHT_MAP")
mod:registerAssetId("maps/material_mask.png", "MATERIAL_MASK")
mod:registerAssetId("maps/rock_density.png", "ROCK_DENSITY_MAP")
mod:registerAssetId("maps/fish_density.png", "FISH_DENSITY_MAP")
mod:registerAssetId("maps/tree_density_map1.png", "TREE_DENSITY_MAP1")
mod:registerAssetId("maps/tree_density_map2.png", "TREE_DENSITY_MAP2")
mod:registerAssetId("maps/tree_density_map4.png", "TREE_DENSITY_MAP4")
mod:registerAssetId("maps/bear_map.png", "BEAR_MAP")
mod:registerAssetId("maps/deer_map.png", "DEER_MAP")
mod:registerAssetId("maps/pheasant_map.png", "PHEASANT_MAP")



-- register custom map
mod:register({
	DataType = "CUSTOM_MAP",
	Id = "MAP_BJORNFELL",
	HeightMap = "HEIGHT_MAP",
	MaterialMask = "MATERIAL_MASK",
	MinHeight = -10,
	MaxHeight = 90,
	VillagePathList = {
		{
			Entrance = { 100, 0, 1013 },
			Exit = { 100, 0, 13 },
		},
		{
			Entrance = { 200, 0, 1013 },
			Exit = { 200, 0, 13 },
		},
		{
			Entrance = { 300, 0, 1013 },
			Exit = { 300, 0, 13 },
		},
		{
			Entrance = { 400, 0, 1013 },
			Exit = { 400, 0, 13 },
		},
		{
			Entrance = { 500, 0, 1013 },
			Exit = { 500, 0, 13 },
		},
		{
			Entrance = { 600, 0, 1013 },
			Exit = { 600, 0, 13 },
		},
		{
			Entrance = { 700, 0, 1013 },
			Exit = { 700, 0, 13 },
		},
		{
			Entrance = { 800, 0, 1013 },
			Exit = { 800, 0, 13 },
		}
	},
	SpawnList = {
		{
			-- creating Mineral Deposit 1
			Prefab = "PREFAB_MINERAL_SOURCE_1",
			Position = { 190 , 15 , 925 },
			Orientation = { 0.0, 120.0 , 0.0},
			PatchBeforeVersion = "1.1.0"
		},
		{
			-- creating Mineral Deposit 2
			Prefab = "PREFAB_MINERAL_SOURCE_1",
			Position = { 262 , 12 , 813 },
			Orientation = { 0.0, -0.0 , 0.0},
			PatchBeforeVersion = "1.1.0"
		},
		{
			-- creating Mineral Deposit 3
			Prefab = "PREFAB_MINERAL_SOURCE_1",
			Position = { 461 , 13 , 622 },
			Orientation = { 0.0, -0.0 , 0.0},
			PatchBeforeVersion = "1.1.0"
		},
		{
			-- creating Mineral Deposit 4
			Prefab = "PREFAB_MINERAL_SOURCE_1",
			Position = { 264 , 20 , 540 },
			Orientation = { 0.0, -0.0 , 0.0},
			PatchBeforeVersion = "1.1.0"
		},
		{
			-- creating Mineral Deposit 5
			Prefab = "PREFAB_MINERAL_SOURCE_1",
			Position = { 81 , 15 , 635 },
			Orientation = { 0.0, 100.0 , 0.0},
			PatchBeforeVersion = "1.1.0"
		},
		{
			-- creating Mineral Deposit 6
			Prefab = "PREFAB_MINERAL_SOURCE_1",
			Position = { 235 , 14 , 323 },
			Orientation = { 0.0, -170.0 , 0.0},
			PatchBeforeVersion = "1.1.0"
		},
		{
			-- creating Mineral Deposit 7
			Prefab = "PREFAB_MINERAL_SOURCE_1",
			Position = { 117 , 19 , 138 },
			Orientation = { 0.0, 40.0 , 0.0},
			PatchBeforeVersion = "1.1.0"
		},
		{
			-- creating Mineral Deposit 8
			Prefab = "PREFAB_MINERAL_SOURCE_1",
			Position = { 403 , 18 , 184 },
			Orientation = { 0.0, -90.0 , 0.0},
			PatchBeforeVersion = "1.1.0"
		},
		{
			-- creating Mineral Deposit 9
			Prefab = "PREFAB_MINERAL_SOURCE_1",
			Position = { 352 , 16 , 449 },
			Orientation = { 0.0, -170.0 , 0.0},
			PatchBeforeVersion = "1.1.0"
		},
		{
			-- creating Mineral Deposit 10
			Prefab = "PREFAB_MINERAL_SOURCE_1",
			Position = { 590 , 14 , 141 },
			Orientation = { 0.0, 140.0 , 0.0},
			PatchBeforeVersion = "1.1.0"
		},
		{
			-- creating Mineral Deposit 11
			Prefab = "PREFAB_MINERAL_SOURCE_1",
			Position = { 462 , 9 , 878 },
			Orientation = { 0.0, 140.0 , 0.0},
			PatchBeforeVersion = "1.1.0"
		}
	},
	DensitySpawnList = {
			{
			-- creating Tree Density Map1
			DensityMap = "TREE_DENSITY_MAP1",
			Density = 0.8,
			PrefabConfigList = {
				{
						PrefabList = { "PREFAB_TREE_OAK" },
						RandomWeight = 3.0,
						OffsetSizeRange = {
							Min = 0.75,
							Max = 1.0
					},
						OrientationRange = {
							Min = { 0, -180, 0 },
							Max = { 0, 180, 0 }
					},
						ScaleRange = {
							Min = 1.5,
							Max = 2.0
					},
						ColorRange = {
							Min = { 0.8,0.8,0.8,1 },
							Max = { 1,1,1,1 }
					}
				},
				{
						PrefabList = { "PREFAB_TREE_POPLAR" },
						RandomWeight = 3.0,
						OffsetSizeRange = {
							Min = 0.75,
							Max = 1.0
					},
						OrientationRange = {
							Min = { 0, -180, 0 },
							Max = { 0, 180, 0 }
					},
						ScaleRange = {
							Min = 1.3,
							Max = 1.7
					},
						ColorRange = {
							Min = { 0.8,0.8,0.8,1 },
							Max = { 1,1,1,1 }
					}
				}
			}
		},
			{
			-- creating Tree Density Map2
			DensityMap = "TREE_DENSITY_MAP2",
			Density = 0.8,
			PrefabConfigList = {
				{
						PrefabList = { "PREFAB_TREE_SYCAMORE" },
						RandomWeight = 3.0,
						OffsetSizeRange = {
							Min = 0.75,
							Max = 1.0
					},
						OrientationRange = {
							Min = { 0, -180, 0 },
							Max = { 0, 180, 0 }
					},
						ScaleRange = {
							Min = 1.0,
							Max = 1.4
					},
						ColorRange = {
							Min = { 0.8,0.8,0.8,1 },
							Max = { 1,1,1,1 }
					}
				},
				{
						PrefabList = { "PREFAB_TREE_POPLAR" },
						RandomWeight = 3.0,
						OffsetSizeRange = {
							Min = 0.75,
							Max = 1.0
					},
						OrientationRange = {
							Min = { 0, -180, 0 },
							Max = { 0, 180, 0 }
					},
						ScaleRange = {
							Min = 0.8,
							Max = 1.2
					},
						ColorRange = {
							Min = { 0.8,0.8,0.8,1 },
							Max = { 1,1,1,1 }
					}
				},
				{
						PrefabList = { "PREFAB_TREE_OAK" },
						RandomWeight = 3.0,
						OffsetSizeRange = {
							Min = 0.75,
							Max = 1.0
					},
						OrientationRange = {
							Min = { 0, -180, 0 },
							Max = { 0, 180, 0 }
					},
						ScaleRange = {
							Min = 1.5,
							Max = 2.0
					},
						ColorRange = {
							Min = { 0.8,0.8,0.8,1 },
							Max = { 1,1,1,1 }
					}
				}
			}
		},
			{
			-- creating Tree Density Map4
			DensityMap = "TREE_DENSITY_MAP4",
			Density = 0.8,
			PrefabConfigList = {
				{
						PrefabList = { "PREFAB_TREE_PINE" },
						RandomWeight = 3.0,
						OffsetSizeRange = {
							Min = 0.75,
							Max = 1.0
					},
						OrientationRange = {
							Min = { 0, -180, 0 },
							Max = { 0, 180, 0 }
					},
						ScaleRange = {
							Min = 1.2,
							Max = 1.8
					},
						ColorRange = {
							Min = { 0.8,0.8,0.8,1 },
							Max = { 1,1,1,1 }
					}
				}
			}
		},
		{
			-- Create rock outcrops - stone resource
			DensityMap = "ROCK_DENSITY_MAP",
			Density =0.9,
			PrefabConfigList = {
				{
						PrefabList = { "PREFAB_RESOURCE_ROCK" },
						RandomWeight = 0.1,
						OffsetSizeRange = {
							Min = 0.75,
							Max = 1.0
					},
						OrientationRange = {
							Min = { 0, -180, 0 },
							Max = { 0, 180, 0 }
					},
						ScaleRange = {
							Min = 0.85,
							Max = 1.15
					},
						ColorRange = {
							Min = { 0.8, 0.8, 0.8, 1 },
							Max = { 1, 1, 1, 1 }
					}
				}
			}
		},
		{
			-- Create fish shoals
			DensityMap = "FISH_DENSITY_MAP",
			Density =0.9,
			PrefabConfigList = {
				{
						PrefabList = { "PREFAB_RESOURCE_FISH" },
						RandomWeight = 0.1,
						OffsetSizeRange = {
							Min = 0.75,
							Max = 1.0
					},
						OrientationRange = {
							Min = { 0, -180, 0 },
							Max = { 0, 180, 0 }
					},
						ScaleRange = {
							Min = 0.85,
							Max = 1.15
					},
						ColorRange = {
							Min = { 0.8, 0.8, 0.8, 1 },
							Max = { 1, 1, 1, 1 }
					}
				}
			}
		},
		{
			-- Create Bear for hunting
			DensityMap = "BEAR_MAP",
			Density = 0.9,
			PrefabConfigList = {
				{
						PrefabList = { "PREFAB_HUNT_BEAR" },
						RandomWeight = 0.1,
						OffsetSizeRange = {
							Min = 0.75,
							Max = 1.0
					},
						OrientationRange = {
							Min = { 0, -180, 0 },
							Max = { 0, 180, 0 }
					},
						ScaleRange = {
							Min = 0.85,
							Max = 1.15
					},
						ColorRange = {
							Min = { 0.8, 0.8, 0.8, 1 },
							Max = { 1, 1, 1, 1 }
					}
				}
			}
		},
		{
			-- Create Deer for hunting
			DensityMap = "DEER_MAP",
			Density = 0.9,
			PrefabConfigList = {
				{
						PrefabList = { "PREFAB_HUNT_DEER" },
						RandomWeight = 0.1,
						OffsetSizeRange = {
							Min = 0.75,
							Max = 1.0
					},
						OrientationRange = {
							Min = { 0, -180, 0 },
							Max = { 0, 180, 0 }
					},
						ScaleRange = {
							Min = 1.0,
							Max = 1.1
					},
						ColorRange = {
							Min = { 0.8, 0.8, 0.8, 1 },
							Max = { 1, 1, 1, 1 }
					}
				}
			}
		},
		{
			-- Create Pheasant for hunting
			DensityMap = "PHEASANT_MAP",
			Density = 0.9,
			PrefabConfigList = {
				{
						PrefabList = { "PREFAB_HUNT_PHEASANT" },
						RandomWeight = 0.1,
						OffsetSizeRange = {
							Min = 0.75,
							Max = 1.0
					},
						OrientationRange = {
							Min = { 0, -180, 0 },
							Max = { 0, 180, 0 }
					},
						ScaleRange = {
							Min = 1.0,
							Max = 1.1
					},
						ColorRange = {
							Min = { 0.8, 0.8, 0.8, 1 },
							Max = { 1, 1, 1, 1 }
					}
				}
			}
		}
	}
})
-- This map was created with the help of the UFME
